#ifndef _MOD_FREI0R_H_
#define _MOD_FREI0R_H_

#include <frei0r.h>
#include <glob.h>

typedef int f0r_init_t(void);
typedef void f0r_deinit_t(void);
typedef void f0r_get_plugin_info_t(f0r_plugin_info_t *info);
typedef void f0r_get_param_info_t(f0r_param_info_t* info, int param_index);
typedef f0r_instance_t f0r_construct_t(unsigned int width, unsigned int height);
typedef void f0r_destruct_t(f0r_instance_t instance);
typedef void f0r_set_param_value_t(f0r_instance_t instance, f0r_param_t param, int param_index);
typedef void f0r_get_param_value_t(f0r_instance_t instance, f0r_param_t param, int param_index);
typedef void f0r_update_t(
    f0r_instance_t instance,
    double time,
    const uint32_t* inframe,
    uint32_t* outframe
    );
typedef void f0r_update2_t(
    f0r_instance_t instance,
    double time,
    const uint32_t* inframe1,
    const uint32_t* inframe2,
    const uint32_t* inframe3,
    uint32_t* outframe
    );

typedef struct frei0r_plugin
    {
    void *dll;
    int init;
    f0r_plugin_info_t info;
    f0r_param_info_t *param_info;
    f0r_init_t *f_init;
    f0r_deinit_t *f_deinit;
    f0r_get_plugin_info_t *f_get_plugin_info;
    f0r_get_param_info_t *f_get_param_info;
    f0r_construct_t *f_construct;
    f0r_destruct_t *f_destruct;
    f0r_set_param_value_t *f_set_param_value;
    f0r_get_param_value_t *f_get_param_value;
    f0r_update_t *f_update;
    f0r_update2_t *f_update2;
    } frei0r_plugin_t;

typedef struct frei0r_list
    {
    int count;
    int mixer_count;
    int filter_count;
    frei0r_plugin_t **fp;
    } frei0r_list_t;

frei0r_list_t *frei0r_get_plugins(void);
void frei0r_free_plugins(frei0r_list_t *fl);
void frei0r_glob_all(glob_t *g);
frei0r_plugin_t *frei0r_load(char *path);
void frei0r_close(frei0r_plugin_t *fp);
frei0r_plugin_t *frei0r_search(frei0r_list_t const *fl, char const *name);
int frei0r_match(frei0r_list_t const *fl, char const *mask, int i);

#endif /* _MOD_FREI0R_H_ */
